import cn.estsh.i3plus.ext.mes.api.base.IMesPartSapService
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack
import cn.estsh.i3plus.pojo.mes.bean.MesCustomerCarModel
import cn.estsh.i3plus.pojo.mes.bean.MesPartProdGroup
import cn.estsh.i3plus.pojo.mes.bean.MesPartSap
import cn.estsh.i3plus.pojo.mes.bean.MesProductVersion
import cn.estsh.i3plus.pojo.mes.dbinterface.MesInterfaceDataMapper
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerCarModelRepository
import cn.estsh.i3plus.pojo.mes.repository.MesPartProdGroupRepository
import cn.estsh.i3plus.pojo.mes.repository.MesProductVersionRepository
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.util.StringUtils

/**
 * @Description : 工单同步
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/7/12 12:28
 * @Modify:
 * */
class MesToPiscesWorkOrder {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesToPiscesWorkOrder.class)

    @Autowired
    private MesPartProdGroupRepository mesPartProdGroupRepository;

    @Autowired
    private MesProductVersionRepository mesProductVersionRepository;

    @Autowired
    private IMesPartSapService mesPartSapService;

    @Autowired
    private MesCustomerCarModelRepository mesCustomerCarModelRepository;

    def filterData(MesInterfaceDataMapper mapper, List<Map<String, Object>> srcData) throws Exception {
        if (srcData == null || srcData.size() == 0) {
            return srcData
        }
        String organizeCode = mapper.getOrganizeCode();
        for (Map<String, Object> rowMap : srcData) {
            if (!StringUtils.isEmpty(rowMap.get("PART_NO")) && !StringUtils.isEmpty(rowMap.get("PRODUCT_VERSION")) && !StringUtils.isEmpty(rowMap.get("ERP_WORK_CENTER"))) {
                MesProductVersion mesProductVersion = getMesProductVersion(organizeCode, String.valueOf(rowMap.get(rowMap.get("PART_NO"))), String.valueOf(rowMap.get("PRODUCT_VERSION")), String.valueOf(rowMap.get("ERP_WORK_CENTER")))
                if (!Objects.isNull(mesProductVersion)) {
                    rowMap.put("BOM_CODE", mesProductVersion.getAlternativePartList())
                } else {
                    LOGGER.info("物料{}生产版本{}工作中心{}，信息不存在", String.valueOf(rowMap.get(rowMap.get("PART_NO"))), String.valueOf(rowMap.get(rowMap.get("PRODUCT_VERSION"))), String.valueOf(rowMap.get(rowMap.get("ERP_WORK_CENTER"))))
                }
            }
            //获取物料信息
            if (!StringUtils.isEmpty(rowMap.get("PART_NO"))) {
                try {
                    MesPartSap mesPartSap = mesPartSapService.getMesPartSapByPartNo(String.valueOf(rowMap.get(rowMap.get("PART_NO"))), organizeCode)
                    rowMap.put("UOM", mesPartSap.getUnit())
                } catch (Exception e) {
                    LOGGER.info("物料{},信息不存在", String.valueOf(rowMap.get("PART_NO")))
                }
            }
            //获取零件生产组
            if (!StringUtils.isEmpty(rowMap.get("PART_PROD_GROUP_CODE"))) {
                MesPartProdGroup mesPartProdGroup = getMesPartProdGroup(organizeCode, String.valueOf(rowMap.get(rowMap.get("PART_PROD_GROUP_CODE"))));
                if (!Objects.isNull(mesPartProdGroup)) {
                    rowMap.put("PRODUCT_GROUP_ID", mesPartProdGroup.getId())
                } else {
                    LOGGER.info("零件生产组{}信息不存在", String.valueOf(rowMap.get("PART_PROD_GROUP_CODE")))
                }
            }

            //获取车型名称
            if (!StringUtils.isEmpty(rowMap.get("CAR_MODEL_CODE"))) {
                MesCustomerCarModel mesCustomerCarModel = getMesCustomerCarModel(organizeCode, String.valueOf(rowMap.get(rowMap.get("CAR_MODEL_CODE"))))
                if (!Objects.isNull(mesCustomerCarModel)) {
                    rowMap.put("VEHICLE_NAME", mesCustomerCarModel.getCarModelName())
                } else {
                    LOGGER.info("车型代码{}信息不存在", String.valueOf(rowMap.get("CAR_MODEL_CODE")))
                }
            }
        }
        return srcData;
    }

    private MesProductVersion getMesProductVersion(String organizeCode, String partNo, String workCenterCode, String productVersion) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partNo, "partNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(workCenterCode, "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(productVersion, "productVersion", ddlPackBean);
        return mesProductVersionRepository.getByProperty(ddlPackBean);
    }

    private MesPartProdGroup getMesPartProdGroup(String organizeCode, String partProdGroupCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partProdGroupCode, "partProdGroupCode", ddlPackBean);
        return mesPartProdGroupRepository.getByProperty(ddlPackBean);
    }

    private MesCustomerCarModel getMesCustomerCarModel(String organizeCode, String carModelCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(carModelCode, "carModelCode", ddlPackBean);
        return mesCustomerCarModelRepository.getByProperty(ddlPackBean);
    }
}